#include "VideoSteuerung.h"
#include "../../Initialisierung/Initialisierung.h"
#include <DateiSystem.h>
#include <Text.h>
#include <Punkt.h>
#include <Bildschirm.h>
#include <MausEreignis.h>

// Inhalt der VideoSteuerung Klasse aus VideoSteuerung.h
// Konstruktor
VideoSteuerung::VideoSteuerung( Schrift *zSchrift, Bildschirm *zScreen )
{
	pos = zScreen->getBackBufferSize() - Punkt( 285, 30 );
	verlassen = initKnopf( 0, 0, 100, 22, zSchrift, Knopf::Style::Sichtbar, "Verlassen" );
	initToolTip( verlassen, "Aufzeichnung verlassen", zSchrift->getThis(), zScreen );
	langsamer = initKnopf( 105, 0, 22, 22, 0, 0, "" );
	langsamer->setStyle( Knopf::Style::Sichtbar | Knopf::Style::Erlaubt | Knopf::Style::HAlpha | Knopf::Style::HBild | Knopf::Style::Hintergrund | Knopf::Style::KlickBuffer | Knopf::Style::Rahmen );
	langsamer->setLinienRahmenBreite( 1 );
	stop = initKnopf( 130, 0, 22, 22, 0, 0, "" );
	stop->setStyle( Knopf::Style::Sichtbar | Knopf::Style::Erlaubt | Knopf::Style::HAlpha | Knopf::Style::HBild | Knopf::Style::Hintergrund | Knopf::Style::KlickBuffer | Knopf::Style::Rahmen );
	stop->setLinienRahmenBreite( 1 );
	schneller = initKnopf( 155, 0, 22, 22, 0, 0, "" );
	schneller->setStyle( Knopf::Style::Sichtbar | Knopf::Style::Erlaubt | Knopf::Style::HAlpha | Knopf::Style::HBild | Knopf::Style::Hintergrund | Knopf::Style::KlickBuffer | Knopf::Style::Rahmen );
	schneller->setLinienRahmenBreite( 1 );
	LTDBDatei *vBilder = new LTDBDatei();
	vBilder->setDatei( new Text( "data/client/bilder/video.ltdb" ) );
	vBilder->leseDaten( 0 );
	langsamer->setHintergrundBildZ( vBilder->laden( 0, new Text( "zurck.png" ) ) );
	stop->setHintergrundBildZ( vBilder->laden( 0, new Text( "pause.png" ) ) );
	schneller->setHintergrundBildZ( vBilder->laden( 0, new Text( "vor.png" ) ) );
	vBilder->release();
	initToolTip( langsamer, "Langsammer abspielen", zSchrift->getThis(), zScreen );
	initToolTip( stop, "Pause", zSchrift->getThis(), zScreen );
	initToolTip( schneller, "Schneller abspielen", zSchrift->getThis(), zScreen );
	zeit = initTextFeld( 180, 0, 100, 22, zSchrift, TextFeld::Style::Text | TextFeld::Style::Center, "0:0:0" );
	spielZeit = 0;
	speed = 1;
	vorwrts = 1;
	zeitlupe = 0;
	beendet = 0;
	bJetzt = 0;
	ref = 1;
}

// Destruktor
VideoSteuerung::~VideoSteuerung()
{
	verlassen->release();
	langsamer->release();
	stop->release();
	schneller->release();
	zeit->release();
}

// Nicht constant
void VideoSteuerung::doMausEreignis( MausEreignis &me )
{
	me.mx -= pos.x;
	me.my -= pos.y;
	bool vera = me.verarbeitet;
	verlassen->doMausEreignis( me );
	if( !vera && me.verarbeitet )
	{
		if( me.id == ME_RLinks )
		{
			bJetzt = 1;
			beendet = 1;
		}
		vera = 1;
	}
	langsamer->doMausEreignis( me );
	if( !vera && me.verarbeitet )
	{
		if( me.id == ME_RLinks )
		{
			if( !speed )
			{
				vorwrts = 0;
				speed = 1;
				stop->addStyle( Knopf::Style::Erlaubt );
				schneller->addStyle( Knopf::Style::Erlaubt );
			}
			else
			{
				speed += ( vorwrts && zeitlupe ) || ( !vorwrts && !zeitlupe );
				speed -= ( vorwrts && !zeitlupe ) || ( !vorwrts && zeitlupe );
				zeitlupe |= vorwrts && ( speed == 0 );
				zeitlupe &= vorwrts || ( speed != 0 );
				speed += speed == 0;
			}
		}
		vera = 1;
	}
	stop->doMausEreignis( me );
	if( !vera && me.verarbeitet )
	{
		if( me.id == ME_RLinks )
		{
			speed = 0;
			zeitlupe = 0;
			stop->removeStyle( Knopf::Style::Erlaubt );
		}
		vera = 1;
	}
	schneller->doMausEreignis( me );
	if( !vera && me.verarbeitet )
	{
		if( me.id == ME_RLinks )
		{
			if( !speed )
			{
				vorwrts = 1;
				speed = 1;
				stop->addStyle( Knopf::Style::Erlaubt );
				langsamer->addStyle( Knopf::Style::Erlaubt );
			}
			else
			{
				speed += ( vorwrts && !zeitlupe ) || ( !vorwrts && zeitlupe );
				speed -= ( vorwrts && zeitlupe ) || ( !vorwrts && !zeitlupe );
				zeitlupe |= !vorwrts && ( speed == 0 );
				zeitlupe &= !vorwrts || ( speed != 0 );
				speed += speed == 0;
			}
		}
		vera = 1;
	}
	me.mx += pos.x;
	me.my += pos.y;
}

bool VideoSteuerung::tick( double zeit )
{
	bool ret = langsamer->tick( zeit );
	ret |= verlassen->tick( zeit );
	ret |= stop->tick( zeit );
	ret |= schneller->tick( zeit );
	spielZeit += zeit;
	if( (int)spielZeit != (int)( spielZeit - zeit ) )
	{
		Text z;
		z = (int)spielZeit / 60 / 60;
		z += ":";
		z += (int)spielZeit / 60 % 60;
		z += ":";
		z += (int)spielZeit % 60;
		this->zeit->setText( z );
	}
	ret |= this->zeit->tick( zeit );
	return ret;
}

void VideoSteuerung::render( Bild &zRObj )
{
	if( !zRObj.setDrawOptions( pos.x, pos.y, 300, 30 ) )
		return;
	verlassen->render( zRObj );
	langsamer->render( zRObj );
	stop->render( zRObj );
	schneller->render( zRObj );
	zeit->render( zRObj );
	zRObj.releaseDrawOptions();
}

void VideoSteuerung::pause( bool ende )
{
	speed = 0;
	zeitlupe = 0;
	stop->removeStyle( Knopf::Style::Erlaubt );
	if( ende )
		schneller->removeStyle( Knopf::Style::Erlaubt );
	else
		langsamer->removeStyle( Knopf::Style::Erlaubt );
}

bool VideoSteuerung::istBeendet( bool jetzt )
{
	if( jetzt )
	{
		if( bJetzt )
		{
			bJetzt = 0;
			return 1;
		}
		return 0;
	}
	return beendet;
}

// constant
int VideoSteuerung::getSpeed() const
{
	return speed + zeitlupe;
}

bool VideoSteuerung::istVorwrts() const
{
	return vorwrts;
}

bool VideoSteuerung::istZeitlupe() const
{
	return zeitlupe;
}

// Reference Counting
VideoSteuerung *VideoSteuerung::getThis()
{
	ref++;
	return this;
}

VideoSteuerung *VideoSteuerung::release()
{
	ref--;
	if( !ref )
		delete this;
	return 0;
}